#!/tmp/.ZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZpErLZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZperl/bin/perl -w
use strict;
use warnings;
use PureMessage::Manager::Request qw($REQUEST %in);
use PureMessage::MessageLog::Watch::Config;
use PureMessage::Manager::Response::Services;
use Data::Dumper;
eval {

my $r = $REQUEST->get_response('PureMessage::Manager::Response::Services',
			       'stderr_to_stdout' => 1);
$r->title('Perimeter Protection Options');
$r->add_taskmenu(
    { 
	title   => 'Perimeter Protection',
	buttons => [[ {type=>'text',text=>"Edit Log Watch Options", href=>"/Services/perimeter.cgi"} ]]
    }
);

$r->add_help(text => "Help", tab => "locserv", page => "mgr_locserv_per" );

my $conf = PureMessage::MessageLog::Watch::Config->new();
my %data = $conf->as_hash;
#print Dumper \%data;

# save the request if the save action was called
if ($REQUEST->query('a') && $REQUEST->query('a') eq 'save') {
    eval {_save($REQUEST, $conf, %in)};
    if ($@) {
	if ($@ =~ /^Input error/) {
	    $@ =~ s/(.*)\bat .*/$1/;
	}
	$r->raise_error('Error saving options. ' . $@);
    }
    else {
	$r->print_notification('Log watch options saved OK.');
    }
    %data = %in;
}

    

my @buttons = (
	       { type=>'icon',icon=>'save',text=>'Save', name=>'save', value=>'save'},
	       { type=>'icon',icon=>'cancel',text=>'Cancel', href=>'index.cgi', alt=>'cancel'}
	       );

my %state = ();
my $tb = $r->ToolBar(
                     name => 'tb',
		     buttons => [\@buttons ],
		     width=>600,
		     state => \%state,
		     );

my ($fields, $controls) = get_ui(%data);
#print Dumper $controls;
my $f = $r->Form( 
		  'data'     => $controls,
		  'fields'   => $fields,
		  'title'    => 'Log Watch Options', 
		  'toolbar'  => $tb,
		  'width'    => 600,
		  'columns'  => 2,
		  'action'   => 'perimeter.cgi?a=save',
		  );

$r->add_part($f,'body_form');

$r->send;
};
print "Error: $@\n" if $@;

sub get_ui {
    my %data = @_;

    my $f = [ # fields
	      'Scan Window (min)', 'Relays',
	      'Max Lines', 'Senders',
	      'Max Recipients', 'Recipients',       
	      'Max Message Size (MB)', 'Message Size',
	     ];

    my $c = { # controls
	'Scan Window (min)',     _txt('scan_window', $data{'scan_window'} || 30),
	'Max Lines',             _txt('max_lines',   $data{'max_lines'}   || 10000),
	'Max Recipients' ,      _txt('max_rcpts',   $data{'max_rcpts'}   || 50),
	'Max Message Size (MB)', _txt('max_size',    $data{'max_size'}    || 10),
	'High Water Marks',      _blank(),
	'Relays',                _txt('hw_relay',    $data{'hw_relay'}    || 5000),
	'Senders',               _txt('hw_senders',  $data{'hw_senders'}  || 5000),
	'Recipients',            _txt('hw_rcpts',    $data{'hw_rcpts'}    || 5000),
	'Message Size',          _txt('hw_size',     $data{'hw_size'}     || 5000),
    };		
    return ($f, $c);
}

sub _txt($;$) {
    my ($name, $val) = @_;
    $val = '' unless $val;
    my $str = '<input type="text" class="stretch" name="';
    $str .= $name . '" value="' . $val . '">';
    return $str;
}

sub _blank() { '&nbsp;' }

sub _save {
    my ($req, $conf, %qry) = @_;

    $req->typedecl('#bignum', sub { 
	die 'Value must be no more one billion' if $_[3] > 1000000000 } );

    $req->typedecl('#smallnum', sub { 
	die 'Value must be no more than one thousand' if $_[3] > 1000 } );

    $req->typedecl('#pos', sub { 
	die 'Value cannot be negative' if $_[3] < 0 } );
    
    $req->typedecl("scan_window","#value #int #bignum #pos",   "Scan Window");
    $req->typedecl("max_lines",  "#value #int #bignum #pos",   "Max Lines");
    $req->typedecl("max_rcpts",  "#value #int #smallnum #pos", "Max Recipients" );
    $req->typedecl("max_size",   "#value #num #smallnum #pos", "Max Message Size");
    $req->typedecl("hw_relay",   "#value #int #bignum #pos",   "High Water Mark, Relays" );
    $req->typedecl("hw_senders", "#value #int #bignum #pos",   "High Water Mark, Senders" );
    $req->typedecl("hw_rcpts",   "#value #int #bignum #pos",   "High Water Mark, Recipients" );
    $req->typedecl("hw_size",    "#value #int #bignum #pos",   "HIgh Water Mark, Message Size" );

    my @errs = $req->validate();
    die join("<br>\n", @errs) if scalar @errs;

    foreach my $k (qw(scan_window max_lines max_rcpts max_size 
		      hw_relay hw_senders hw_rcpts hw_size)) {
	$conf->val($k, $qry{$k}) if $qry{$k};
    }
    $conf->save();

}
